#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-

"A very simple example application."

import WebStack.Generic
import XSLForms.Resources.WebResources
import XSLForms.Utils
import os

# Site map imports.

from WebStack.Resources.ResourceMap import MapResource
from WebStack.Resources.Selectors import EncodingSelector, PathSelector
from WebStack.Resources.Static import DirectoryResource, StringResource
from XSLForms.Resources.OpenIDLogin import OpenIDLoginResource, Authenticator
from XSLForms.Resources.OpenIDRedirect import OpenIDRedirectResource, OpenIDRedirectAuthenticator
from XSLForms.Resources.OpenIDInitiation import OpenIDInitiationResource

# Configuration settings.

encoding = "utf-8"

# Resource classes.

class VerySimpleResource(XSLForms.Resources.WebResources.XSLFormsResource):

    "A very simple resource providing a hierarchy of editable fields."

    resource_dir = os.path.join(os.path.split(__file__)[0], "Resources")
    template_resources = {
        #"structure" : ("structure_template.xhtml", "structure_output.xsl")
        #"structure" : ("structure_multivalue_template.xhtml", "structure_output.xsl")
        "structure" : ("structure_multivalue_label_template.xhtml", "structure_output.xsl")
        }
    init_resources = {
        #"structure" : ("structure_template.xhtml", "structure_input.xsl")
        #"structure" : ("structure_multivalue_template.xhtml", "structure_input.xsl")
        "structure" : ("structure_multivalue_label_template.xhtml", "structure_input.xsl")
        }
    transform_resources = {
        "comments" : ["structure_comments.xsl"]
        }
    document_resources = {
        #"types" : "structure_types.xml"
        "types" : "structure_types_label.xml",
        "translations" : "translations.xml"
        }
    in_page_resources = {
        "comments" : ("structure", "structure_output_comments.xsl", "comment-node")
        }

    def respond_to_form(self, trans, form):

        """
        Respond to a request having the given transaction 'trans' and the given
        'form' information.
        """

        in_page_resource = self.get_in_page_resource(trans)
        parameters = form.get_parameters()
        documents = form.get_documents()

        # Ensure the presence of a document.

        if documents.has_key("structure"):
            structure = documents["structure"]
        else:
            structure = form.new_instance("structure")

        # Add and remove elements according to the selectors found.

        selectors = form.get_selectors()
        XSLForms.Utils.remove_elements(selectors.get("remove2"))
        XSLForms.Utils.add_elements(selectors.get("add2"), "subitem")
        XSLForms.Utils.remove_elements(selectors.get("remove"))
        XSLForms.Utils.add_elements(selectors.get("add"), "item")

        # Initialise the document, adding enumerations/ranges.

        structure_xsl = self.prepare_initialiser("structure")
        types_xml = self.prepare_document("types")
        structure = self.get_result([structure_xsl], structure, references={"type" : types_xml})

        # Add the comments.

        comments_xsl_list = self.prepare_transform("comments")
        structure = self.get_result(comments_xsl_list, structure)

        # Start the response.

        trans.set_content_type(WebStack.Generic.ContentType("application/xhtml+xml", encoding))

        # Ensure that an output stylesheet exists.

        if in_page_resource in self.in_page_resources.keys():
            trans_xsl = self.prepare_fragment(in_page_resource)
            stylesheet_parameters = self.prepare_parameters(parameters)
        else:
            trans_xsl = self.prepare_output("structure")
            stylesheet_parameters = {}

        # Complete the response.

        try:
            stylesheet_parameters["locale"] = trans.get_content_languages()[0]
        except IndexError:
            pass
        self.send_output(trans, [trans_xsl], structure, stylesheet_parameters,
            references={"translations" : self.prepare_document("translations")})

class InitiationResource(XSLForms.Resources.OpenIDInitiation.OpenIDInitiationResource):

    "Special initiation screen for this application."

    resource_dir = os.path.join(os.path.split(__file__)[0], "Resources")
    template_resources = {
        "initiation" : ("initiation_template.xhtml", "initiation_output.xsl"),
        "success" : ("initiation_success_template.xhtml", "initiation_success_output.xsl")
        }

class LoginResource(XSLForms.Resources.OpenIDLogin.OpenIDLoginResource):

    "Special login screen for this application."

    resource_dir = os.path.join(os.path.split(__file__)[0], "Resources")
    template_resources = {
        "login" : ("login_template.xhtml", "login_output.xsl"),
        "success" : ("success_template.xhtml", "success_output.xsl")
        }

# Site map initialisation.

def get_site(host, secret_key, use_redirect=1, deployed_at=None):

    "Return a simple Web site resource."

    # Configure the user home pages below.

    deployed_at = deployed_at or host

    # Authentication objects.

    credentials = [
        ((deployed_at + "/badger", "badger"), "abc"),
        ((deployed_at + "/vole", "vole"), "xyz"),
        ((deployed_at + u"/", u""), u"")
        ]
    login_authenticator = Authenticator(credentials)
    login_redirect_authenticator = OpenIDRedirectAuthenticator(secret_key, host)

    # Get the main resource and the directory used by the application.

    very_simple_resource = VerySimpleResource()
    directory = very_simple_resource.resource_dir

    # Get some initiation/login resources.

    login_redirect_resource = XSLForms.Resources.OpenIDRedirect.OpenIDRedirectResource(
        host,
        "initiation", # see below
        resource=very_simple_resource,
        authenticator=login_redirect_authenticator,
        logout_parameter_name="logout"
        )
    initiation_resource = InitiationResource(use_redirect=use_redirect)
    login_resource = LoginResource(host, login_authenticator, use_redirect=use_redirect)

    # Make a simple Web site.

    resource = MapResource({
        # Static resources:
        "scripts" : DirectoryResource(os.path.join(directory, "scripts"), {"js" : "text/javascript"}),
        # Initiation resource:
        "initiation" : initiation_resource,
        # Login resource:
        "login" : login_resource,

        # Some local OpenID pages.

        "badger" :
            StringResource("""
                <html>
                  <head>
                    <link rel="openid2.provider openid.server" href="%s/login" />
                    <link rel="openid2.local_id openid.delegate" href="%s/badger" />
                    <title>Badger's Home Page</title>
                  </head>
                  <body>
                    <p>Home page for the OpenID authenticated user, <code>badger</code>.</p>
                  </body>
                </html>
            """ % (deployed_at, deployed_at), WebStack.Generic.ContentType("text/html")),
        u"" :
            StringResource(u"""
                <html>
                  <head>
                    <link rel="openid2.provider openid.server" href="%s/login" />
                    <link rel="openid2.local_id openid.delegate" href="%s/" />
                    <title>Home Page for </title>
                  </head>
                  <body>
                    <p>Home page for the OpenID authenticated user, <code></code>.</p>
                  </body>
                </html>
            """ % (deployed_at, deployed_at), WebStack.Generic.ContentType("text/html", "iso-8859-1")),
        "vole" :
            StringResource("""
                <html>
                  <head>
                    <link rel="openid2.provider openid.server" href="%s/login" />
                    <link rel="openid2.local_id openid.delegate" href="%s/vole" />
                    <title>Vole's Home Page</title>
                  </head>
                  <body>
                    <p>Home page for the OpenID authenticated user, <code>vole</code>.</p>
                  </body>
                </html>
            """ % (deployed_at, deployed_at), WebStack.Generic.ContentType("text/html")),

        # Main page and in-page resources:
        # (Now protected!)
        None : login_redirect_resource
        })

    # Wrap the resource up with information about the application root.

    return EncodingSelector(PathSelector(resource), encoding)

# Resource preparation ahead of time - useful for making installations.

def prepare_resources():
    for cls in [VerySimpleResource, LoginResource, InitiationResource]:
        XSLForms.Resources.WebResources.prepare_resources(cls)

# vim: tabstop=4 expandtab shiftwidth=4
